/*
 * File: Rational.js
 * -----------------
 * This file defines a simple class for representing rational numbers.
 */

/*
 * This class represents a rational number (the quotient of two integers).
 */

function Rational(x, y) {
   if (x === undefined) x = 0;
   if (y === undefined) y = 1;
   let g = gcd(Math.abs(x), Math.abs(y));
   let num = x / g;
   let den = Math.abs(y) / g;
   if (y < 0) num = -num;
   return {
      getNum: function() { return num; },
      getDen: function() { return den; },
      add: function(r) {
              return Rational(num * r.getDen() + r.getNum() * den,
                              den * r.getDen());
           },
      sub: function(r) {
              return Rational(num * r.getDen() - r.getNum() * den,
                              den * r.getDen());
           },
      mul: function(r) {
              return Rational(num * r.getNum(), den * r.getDen());
           },
      div: function(r) {
              return Rational(num * r.getDen(), den * r.getNum());
           },
      toString: function() {
                   if (den === 1) return "" + num;
                   return num + "/" + den;
                }
   };
}

/*
 * Calculates the greatest common divisor using Euclid's algorithm.
 */

function gcd(x, y) {
   let r = x % y;
   while (r !== 0) {
      x = y;
      y = r;
      r = x % y;
   }
   return y;
}
